﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using System.Web.Security;

namespace Demo.Web
{
	public partial class Login : System.Web.UI.Page
	{
		protected void Page_Load(object sender, EventArgs e)
		{
			var txtUserName = Login1.FindControl("UserName") as TextBox;
			if (txtUserName != null)
			{
				txtUserName.Focus();
			}
		}

		protected void Login1_LoggingIn(object sender, LoginCancelEventArgs e)
		{

			// set a session flag if user's password changed since their last login
			// need to do this is LoggingIn event because LastAcitivityDate not updated yet
			// note: this means must update activity when change password 
			var user = Membership.GetUser(Login1.UserName, false);  // don't update activity timestamp

			if (user != null 
				 && ((user.LastPasswordChangedDate > user.LastActivityDate)
					  || (user.CreationDate == user.LastPasswordChangedDate)))
			{
				// temporary password only good for 24 hours, after that account is locked and user must request another password reset
				if (user.LastPasswordChangedDate.AddHours(24) < DateTime.Now)
				{
					LockUserAndLogout(user.UserName);
				}
				else
				{
					// temporary password OK, but set flag to force them to change password
					Session["ForceChangePassword"] = true;
				}
			}
		}
	 
		protected void Login1_LoginError(object sender, EventArgs e)
		{
			MembershipUser user = Membership.GetUser(Login1.UserName);

			if (user != null)
			{
				if (user.IsLockedOut)
					Login1.FailureText = "Your account has been locked out.";
				else if (!user.IsApproved)
					Login1.FailureText = "Your account has not been approved yet. You cannot login until an administrator has approved your account.";
			}
		}

		protected void Login1_Authenticate(object sender, AuthenticateEventArgs e)
		{
			//Before logging the user in, if the user has been inactive for > 90 days, then the account needs to be locked.
			MembershipUser user = Membership.GetUser(Login1.UserName);

			if (Membership.ValidateUser(Login1.UserName, Login1.Password))
			{
				//If the user has been inactive for more than 90 days, then the account needs to be locked.
				//Example: 1/1/2010 + 90 Days = 4/1/2010 < today
				if (user.LastLoginDate.AddDays(90) < System.DateTime.Now)
				{
					LockUserAndLogout(user.UserName);
					return;
				}
				//Require users to change their passwords every 90 days.
				if (user.LastPasswordChangedDate.AddDays(90) < System.DateTime.Now)
					Session["ForceChangePassword"] = true;  // set flag for ChangePassword page

				// if password must be changed then redirect to ChangePassword page
				// Either password > 90 days old, or it was reset since last login
				if (Session["ForceChangePassword"] != null)
				{
					FormsAuthentication.RedirectFromLoginPage(Login1.UserName, false);  // forces auth cookie
					Response.Redirect("~/allusers/ChangePassword.aspx");
				}

				// check if user is going to be redirected to an application that is under maintenance 
				//  or that they're not authorized to view
				string returnUrl = Request.QueryString["ReturnUrl"];
				if (!string.IsNullOrEmpty(returnUrl))
				{
					Uri targetUri;
					if (!Uri.TryCreate(returnUrl, UriKind.Absolute, out targetUri))
					{
						Uri.TryCreate(HttpContext.Current.Request.Url, returnUrl, out targetUri);
					}
				}
				FormsAuthentication.RedirectFromLoginPage(Login1.UserName, false);
			}
		}

		private void LockUserAndLogout(string userName)
		{
			FormsAuthentication.SignOut();
			Session.Abandon();
		}
	}
}
